/// <reference types="node" />
import * as fs from 'fs';
import { Observable } from 'rxjs';
import webpack = require('webpack');
import { BuilderContext } from '@angular-devkit/architect';
import { BuildResult, WebpackLoggingCallback } from '@angular-devkit/build-webpack';
import { logging, virtualFs } from '@angular-devkit/core';
import { BrowserBuilderOutput } from '@angular-devkit/build-angular/src/browser';
import { ServerBuilderOutput } from '@angular-devkit/build-angular/src/server';
import { BrowserBuilderInitContext, BrowserBuilderOptions, ServerBuilderInitContext, ServerBuilderOptions, UniversalBuildOptions, UniversalCompilationOutput, UniversalTargetOptionsMap } from './types';
export declare const cacheDownlevelPath: string | undefined;
export declare function getUniversalTargetOptions<K extends keyof UniversalTargetOptionsMap>(context: BuilderContext, options: UniversalBuildOptions, platformTarget: K): Promise<UniversalTargetOptionsMap[K]>;
export declare function validateTargetOptions<T>(context: BuilderContext, targetString: string, overrides?: Partial<T>): Promise<T>;
export declare function initializeBrowserBuilder(universalOptions: UniversalBuildOptions, browserOptions: BrowserBuilderOptions, context: BuilderContext, host: virtualFs.Host<fs.Stats>): Promise<BrowserBuilderInitContext>;
export declare function createBrowserBuilderFinalizer(context: BuilderContext, host: virtualFs.Host<fs.Stats>, { options, projectRoot, projectSourceRoot, i18n, isDifferentialLoadingNeeded, target, useBundleDownleveling, buildBrowserFeatures, }: BrowserBuilderInitContext): (startTime: number, buildEvent: BuildResult) => Promise<BrowserBuilderOutput>;
export declare function initializeServerBuilder(universalOptions: UniversalBuildOptions, browserOptions: BrowserBuilderOptions, serverOptions: ServerBuilderOptions, context: BuilderContext): Promise<ServerBuilderInitContext>;
export declare function createServerBuilderFinalizer(context: BuilderContext, { options, i18n, target, }: ServerBuilderInitContext): (output: BuildResult) => Promise<ServerBuilderOutput>;
export declare function createUniversalWebpackConfig(browserConfig: webpack.Configuration, serverConfig: webpack.Configuration): webpack.Configuration[];
export declare function createUniversalCompilationOutput(multiStats: webpack.compilation.MultiStats & {
    hasErrors: () => boolean;
}): UniversalCompilationOutput;
export declare function createWebpackMultiCompiler(getWebpackModule: () => Promise<typeof webpack>, multiConfig: webpack.Configuration[]): Promise<webpack.MultiCompiler>;
export declare function runUniversalWebpackCompiler(multiCompiler: webpack.MultiCompiler, logger: logging.LoggerApi): Observable<UniversalCompilationOutput>;
export declare function adaptWebpackLoggingCallback(logFn: WebpackLoggingCallback): (multiStats: webpack.Stats | webpack.compilation.MultiStats, multiConfig: webpack.Configuration | webpack.Configuration[]) => void;
export declare function createLoggingCallback(logger: logging.LoggerApi, options?: {
    colors?: boolean;
    verbose?: boolean;
}): (multiStats: webpack.Stats | webpack.compilation.MultiStats, multiConfig: webpack.Configuration | webpack.Configuration[]) => void;
export declare function applyWebpackPartialConfig(config: webpack.Configuration, partialTransformConfigPath: string): Promise<webpack.Configuration>;
