/// <reference types="node" />
import * as fs from 'fs';
import webpack = require('webpack');
import { Architect, BuilderContext } from '@angular-devkit/architect';
import { WorkspaceNodeModulesArchitectHost } from '@angular-devkit/architect/node';
import { WebpackLoggingCallback } from '@angular-devkit/build-webpack';
import { json, logging, schema, virtualFs, workspaces } from '@angular-devkit/core';
import { DevContainerAPI, DevContainerArgs, DevContainerConfig, DevContainerFactory, DevContainerRuntime } from '../../../lib/devContainer';
import { BrowserBuilderInitContext, BrowserBuilderOptions, ServerBuilderInitContext, ServerBuilderOptions, UniversalBuildOptions } from '../build';
export declare type NgContainerAPI = DevContainerAPI;
export interface NgContainerConfig extends DevContainerConfig {
    angularProject?: string;
}
export interface NgContainerFactory extends DevContainerFactory {
    (proc: NodeJS.Process): NgContainerAPI;
}
export interface NgContainerArgs extends DevContainerArgs {
    target: string;
    verbose: boolean;
}
export declare const ANUGULAR_CONFIG_NAMES: string[];
export declare class NgContainer extends DevContainerRuntime {
    readonly runtimePath: string;
    readonly proc: NodeJS.Process;
    args: NgContainerArgs;
    readonly architectLogger: logging.LoggerApi;
    firstCompilation: boolean;
    firstCompilationFailed: boolean;
    rebootCalled: boolean;
    webpackLogging: WebpackLoggingCallback;
    angularConfigPath: string;
    registry: schema.SchemaRegistry;
    host: virtualFs.Host<fs.Stats>;
    workspace: workspaces.WorkspaceDefinition;
    architectHost: WorkspaceNodeModulesArchitectHost;
    architect: Architect;
    universalOptions: UniversalBuildOptions;
    browserOptions: BrowserBuilderOptions;
    browserBuilderInitContext: BrowserBuilderInitContext;
    serverOptions: ServerBuilderOptions;
    serverBuilderInitContext: ServerBuilderInitContext;
    builderContext: BuilderContext;
    constructor(runtimePath: string, proc: NodeJS.Process);
    validateTargetOptions<T>(targetString: string, overrides?: Partial<T>): Promise<T>;
    validateBuilderOptions<T>(options: json.JsonObject, builderName: string): Promise<T>;
    bootstrap(): Promise<this>;
    createArchitectLoggerAdapter(architectLogger: logging.LoggerApi): Console;
    getWebpackConfig(): Promise<webpack.Configuration[]>;
    onShutUp(config: DevContainerConfig): Promise<void>;
    printCompilerStats(_config: NgContainerConfig, _multiStats: webpack.Stats): void;
    loadConfig(args: NgContainerArgs): {
        config: NgContainerConfig;
        configPath?: string | undefined;
    };
    parseProcessArgsOptions(): yargs.Options;
}
declare const _default: NgContainerFactory;
export default _default;
