/// <reference types="node" />
import * as http from 'http';
import * as webpack from 'webpack';
import { ContainerAPI, ContainerArgs, ContainerConfig, ContainerFactory, ContainerRuntime } from './util/container';
export { ContainerAPI, ContainerFactory } from './util/container';
export interface DevContainerAPI extends ContainerAPI {
}
export interface DevContainerArgs extends ContainerArgs {
}
export interface DevContainerConfig extends ContainerConfig {
    autoRestart?: boolean;
    hmr?: boolean | string | WebpackContainerHMROptions;
    topModuleEntries: (string | RegExp)[];
    webpackConfig?: string;
    beforeRequireNodeBundle?(compiler: webpack.Compiler, mainOutputPath: string, stats: webpack.Stats): void;
    injectWebpackStats?(compilerStats: {
        [name: string]: webpack.Stats;
    }, req: http.IncomingMessage, res: http.OutgoingMessage): void;
    onBundleAvailable?(bundle: WebpackBundle): void;
    onCompilerWatchRun?(compiler: webpack.Compiler, done: (err?: Error) => void): void;
    onCompilerBeforeCompile?(compiler: webpack.Compiler, compilationParams: any, // tslint:disable-line:no-any
    done: (err?: Error) => void): void;
    onCompilerCompile?(compiler: webpack.Compiler, compilationParams: any): void;
    onCompilerThisCompilation?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation, compilationParams: any): void;
    onCompilerCompilation?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation, compilationParams: any): void;
    onCompilerMake?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation, done: (err?: Error) => void): void;
    onCompilerAfterCompile?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation, done: (err?: Error) => void): void;
    onCompilerShouldEmit?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation): boolean;
    onCompilerEmit?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation, done: (err?: Error) => void): void;
    onCompilerAfterEmit?(compiler: webpack.Compiler, compilation: webpack.compilation.Compilation, done: (err?: Error) => void): void;
    onCompilerDone?(compiler: webpack.Compiler, stats: webpack.Stats, done?: (err?: Error) => void): void;
    onCompilerFailed?(compiler: webpack.Compiler, err: Error): void;
    onCompilerInvalid?(compiler: webpack.Compiler, fileName: string, changeTime: number): void;
    onCompilerWatchClose?(compiler: webpack.Compiler): void;
    prepareWebpackCompiler?(compiler: webpack.Compiler): void;
    prepareWebpackNodeCompiler?(compiler: webpack.Compiler): void;
    prepareWebpackWebCompiler?(compiler: webpack.Compiler): void;
    prepareWebpackConfig?(webpackConfig: webpack.Configuration): void;
    prepareWebpackNodeConfig?(webpackConfig: webpack.Configuration): void;
    prepareWebpackWebConfig?(webpackConfig: webpack.Configuration): void;
    printCompilerStats?(stats: webpack.Stats): void;
    requestDecorator?(bundle: WebpackBundle, req: http.IncomingMessage, res: http.OutgoingMessage, next: (err?: any) => void): void;
}
export interface DevContainerFactory extends ContainerFactory {
    (proc: NodeJS.Process): DevContainerAPI;
}
export interface WebpackContainerHMROptions {
    enable: boolean;
    configs?: string[];
    path?: string;
    entries?: string[];
    entriesNode?: string[];
    entriesWeb?: string[];
    hotPollInterval?: number;
    hotMiddleware?: WebpackHotMiddlewareOptions;
    hotMiddlewareClient?: WebpackHotMiddlewareClientOptions;
    plugin?: WebpackHMRPluginOptions;
    pluginNode?: WebpackHMRPluginOptions;
    pluginWeb?: WebpackHMRPluginOptions;
}
export interface WebpackBundle {
    name: string;
    mainOutputPath: string;
    mainOutputExports: any;
    mainOutputExportsDefault: any;
    compiler: webpack.Compiler;
    stats: webpack.Stats;
}
export interface WebpackHMRPluginOptions {
    multiStep?: boolean;
    fullBuildTimeout?: number;
    requestTimeout?: number;
}
export interface WebpackHotMiddlewareOptions {
    path: string;
    log?: boolean | Console['log'];
    heartbeat?: number;
}
export interface WebpackHotMiddlewareClientOptions {
    path?: string;
    name?: string;
    timeout?: number;
    overlay?: boolean;
    reload?: boolean;
    noInfo?: boolean;
    quiet?: boolean;
    dynamicPublicPath?: boolean;
    autoConnect?: boolean;
    ansiColors?: {
        [colorName: string]: string | [string, string];
    };
    overlayStyles?: {
        [stylePropName: string]: string;
    };
    overlayWarnings?: boolean;
}
export declare const BUNDLE_AVAILABLE_EVENT = "bundleAvailable";
export declare class DevContainerRuntime extends ContainerRuntime {
    bundles: {
        [name: string]: WebpackBundle;
    };
    compiler?: webpack.Compiler | webpack.MultiCompiler;
    compilerStats: {
        [name: string]: webpack.Stats;
    };
    compilerWatching?: webpack.Watching;
    webpackConfig?: webpack.Configuration | webpack.Configuration[];
    emitBundleAvailable(config: DevContainerConfig, stats: webpack.Stats): WebpackBundle | undefined;
    getWebpackConfig(config: DevContainerConfig): Promise<webpack.Configuration | webpack.Configuration[]>;
    parseConfigHMR(config: DevContainerConfig): WebpackContainerHMROptions;
    parseProcessArgs(): DevContainerArgs;
    prepareConfig(config: DevContainerConfig): Promise<void>;
    prepareWebpackCompiler(config: DevContainerConfig, compiler: webpack.Compiler): void;
    prepareWebpackConfig(config: DevContainerConfig, webpackConfig: webpack.Configuration): void;
    printCompilerStats(config: DevContainerConfig, stats: webpack.Stats): void;
    onShutDown(config: DevContainerConfig): void;
    onShutUp(config: DevContainerConfig): Promise<void>;
    tryPrepareUniversalHMR(config: DevContainerConfig, webpackConfig: webpack.Configuration | webpack.Configuration[]): void;
}
declare const _default: DevContainerFactory;
export default _default;
