/// <reference types="node" />
/// <reference types="@_/types/yargs-parser" />
import { ChildProcess } from 'child_process';
import debug = require('debug');
import Watchpack = require('watchpack');
import yargs = require('yargs-parser');
export interface ContainerAPI {
    readonly args: ContainerArgs;
    readonly debug: debug.IDebugger;
    readonly isChild: boolean;
    readonly logger: Console;
    readonly runtimePath: string;
    readonly proc: NodeJS.Process;
    bootstrap(): Promise<ContainerAPI>;
    close(callback?: () => void): void;
    run(callback?: () => void): void;
}
export interface ContainerArgs extends yargs.ArgsParsed {
    config?: string;
    cwd: string;
    debug: boolean;
    require: string[];
}
export interface ContainerConfig {
    context?: string;
    logger: Console;
    metafiles?: string[];
    metadirs?: string[];
    processTitle?: string;
    watchOptions?: Watchpack.WatchOptions;
    bootstrap?(runtime: ContainerRuntime): void | Promise<void>;
    onUp?(runtime: ContainerRuntime): void | Promise<void>;
    onDown?(runtime: ContainerRuntime): void;
}
export interface ContainerFactory {
    (proc: NodeJS.Process): ContainerAPI;
}
export declare const UDKC_CONFIG_NAMES: string[];
export declare const UDKC_PROCESS_TITLE = "udk-ctnr";
export declare class ContainerRuntime implements ContainerAPI {
    readonly runtimePath: string;
    readonly proc: NodeJS.Process;
    static create(runtimePath: string, proc: NodeJS.Process): ContainerAPI;
    static export(mod: NodeJS.Module): ContainerFactory;
    static bootstrapFork(proc: NodeJS.Process): Promise<ContainerAPI>;
    args: ContainerArgs;
    child?: ChildProcess;
    debug: debug.Debugger;
    isChild: boolean;
    logger: Console;
    preloadPaths: string[];
    requirePath?: string;
    watcher?: Watchpack;
    get debugNs(): string;
    constructor(runtimePath: string, proc: NodeJS.Process);
    bootstrap(): Promise<this>;
    close(callback?: () => void): void;
    run(callback?: () => void): void;
    loadConfig(args: ContainerArgs): {
        config: ContainerConfig;
        configPath?: string;
    };
    parseProcessArgs(): ContainerArgs;
    parseProcessArgsOptions(): yargs.Options;
    preloadModule(preloadPath: string): void;
    prepareConfig(config: ContainerConfig): Promise<void>;
    onShutDown(config: ContainerConfig): void;
    onShutUp(config: ContainerConfig): Promise<void>;
}
declare const _default: ContainerFactory;
export default _default;
